@@ -9,7 +9,7 @@ import pyarrow
 
 from . import utils
 from . import stats
-from .utils import ResultsObject
+from .utils import MapResults
 
 
 def read_ray_config():
@@ -98,31 +98,11 @@ def handle_return(out_func, ret, system_stats, system_kwargs, user_kwargs):
               'an unknown number of results lost\n'.format(e), file=sys.stderr)
         traceback.print_exc()
         sys.stderr.flush()
-        progress['failures'] += 1
+        progress.failures += 1
         utils.report_progress(system_kwargs)
         return
 
-    for user_ret, system_ret in ret:
-        if 'result' in user_ret and not isinstance(user_ret['result'], dict) and user_ret['result'] is not None:
-            # fake an exception, make this case look like other failures
-            user_ret['exception'] = "ValueError('user function did not return a dict: {}')".format(
-                repr(user_ret['result']))
-            user_ret['result'] = {}
-        if 'raw_stats' in system_ret:
-            system_stats.combine_stats(system_ret['raw_stats'])
-        pset_id = user_ret['pset']['_pset_id']
-        if 'exception' in user_ret:
-            progress['failures'] += 1
-            progress['exceptions'] += 1
-            system_kwargs['pset_ids'][pset_id]['exception'] = user_ret['exception']
-        else:
-            del system_kwargs['pset_ids'][pset_id]
-            system_kwargs['results'].append(user_ret)
-            progress['finished'] += len(ret)
-        if out_func:
-            out_func(user_ret, system_kwargs, user_kwargs)
-
-    utils.report_progress(system_kwargs)
+    utils.handle_return_common(out_func, ret, system_stats, system_kwargs, user_kwargs)
 
 
 def check_serialized_size(args, factor=1.2):
@@ -183,7 +163,7 @@ def map(func, psets, out_func=None, user_kwargs=None, chdir=None, outfile=None,
     while psets:
         pset_group = utils.get_pset_group(psets, group_size)
         futures.append(do_work_wrapper.remote(func, system_kwargs, user_kwargs, pset_group))
-        progress['started'] += len(pset_group)
+        progress.started += len(pset_group)
 
         # cores and group_size can change within this function
         futures, cores, group_size = progress_until_fewer(futures, cores, factor, out_func, system_stats, system_kwargs, user_kwargs, group_size)
@@ -203,4 +183,4 @@ def map(func, psets, out_func=None, user_kwargs=None, chdir=None, outfile=None,
 
     system_stats.print_histograms(name)
 
-    return ResultsObject(system_kwargs['results'], list(system_kwargs['pset_ids'].values()), system_kwargs['progress'], system_stats)
+    return MapResults(system_kwargs['results'], list(system_kwargs['pset_ids'].values()), system_kwargs['progress'], system_stats)
