@@ -1,42 +1,31 @@
 import time
 import sys
-from collections import defaultdict
 import uuid
 
 from . import stats
 
 
-class ProgressObject(object):
-    def __init__(self, progress):
-        self._progress = progress
+class MapProgress(object):
+    '''Class to track progress of a map()'''
+    # would be perfect as a dataclass, once python 3.7 is our minimum
+    def __init__(self, d={}):
+        self.total = d.get('total', 0)
+        self.started = d.get('started', 0)
+        self.finished = d.get('finished', 0)
+        self.failures = d.get('failures', 0)
+        self.exceptions = d.get('exceptions', 0)
 
-    @property
-    def total(self):
-        return self._progress['total']
-
-    @property
-    def total(self):
-        return self._progress['started']
-
-    @property
-    def finished(self):
-        return self._progress['finished']
-
-    @property
-    def failures(self):
-        return self._progress['failures']
-
-    @property
-    def exceptions(self):
-        return self._progress['exceptions']
+    def __str__(self):
+        return ', '.join([k+': '+str(v) for k, v in vars(self).items()])
 
 
-class ResultsObject(object):
+class MapResults(object):
     '''
     A container object for the outcome of paramsurvey.map()
     '''
     def __init__(self, results, missing, progress, stats):
         self._results = results
+        self._results_flattened = None
         self._missing = missing
         self._progress = progress
         self._stats = stats
@@ -45,32 +34,39 @@ class ResultsObject(object):
     def results(self):
         return self._results
 
+    @property
+    def results_flattened(self):
+        if self._results_flattened is None:
+            self._results_flattened = flatten_results(self._results)
+        return self._results_flattened
+
     @property
     def missing(self):
         return self._missing
 
     @property
     def progress(self):
-        return ProgressObject(self._progress)
+        return self._progress
 
     @property
     def stats(self):
-        # stats.StatsObject
+        # stats.PerfStats
         return self._stats
 
 
 def report_progress(system_kwargs, final=False):
     t = time.time()
-    force = bool(final or system_kwargs.get('verbose', 0) > 1)
+    verbose = system_kwargs['verbose']
+
+    force = bool(final or verbose > 1)
 
     if force or t - system_kwargs['progress_last'] > system_kwargs['progress_dt']:
         system_kwargs['progress_last'] = t
-        print(system_kwargs['name'], 'progress:',
-              ', '.join([k+': '+str(v) for k, v in system_kwargs['progress'].items()]),
+        print(system_kwargs['name'], 'progress:', str(system_kwargs['progress']),
               file=sys.stderr)
 
-        if final and system_kwargs['pset_ids']:
-            print('left-over psets:', file=sys.stderr)
+        if final and verbose and system_kwargs['pset_ids']:
+            print('missing psets:', file=sys.stderr)
             for pset_id, pset in system_kwargs['pset_ids'].items():
                 print(' ', pset, file=sys.stderr)
 
@@ -79,7 +75,7 @@ def report_progress(system_kwargs, final=False):
 
 def remaining(system_kwargs):
     progress = system_kwargs['progress']
-    return progress['started'] - progress.get('retired', 0)
+    return progress.started - progress.finished
 
 
 def get_pset_group(psets, group_size):
@@ -96,7 +92,7 @@ def map_prep(psets, name, chdir, outfile, out_subdirs, verbose, **kwargs):
     print('starting work on', name, file=sys.stderr)
     sys.stderr.flush()
 
-    system_kwargs = {'results': []}
+    system_kwargs = {'progress': MapProgress({'total': len(psets)}), 'results': []}
     if chdir:
         system_kwargs['chdir'] = chdir
     if outfile:
@@ -105,8 +101,7 @@ def map_prep(psets, name, chdir, outfile, out_subdirs, verbose, **kwargs):
         system_kwargs['out_subdirs'] = out_subdirs
     if name:
         system_kwargs['name'] = name
-    if verbose:
-        system_kwargs['verbose'] = verbose
+    system_kwargs['verbose'] = verbose or 0
 
     if 'raise_in_wrapper' in kwargs:
         system_kwargs['raise_in_wrapper'] = kwargs['raise_in_wrapper']
@@ -114,27 +109,19 @@ def map_prep(psets, name, chdir, outfile, out_subdirs, verbose, **kwargs):
     psets, pset_ids = make_pset_ids(psets)
     system_kwargs['pset_ids'] = pset_ids
 
-    system_stats = stats.StatsObject()
-    progress = defaultdict(int)
-    progress['total'] = len(psets)
-    system_kwargs['progress'] = progress
+    system_stats = stats.PerfStats()
     system_kwargs['progress_last'] = 0.
     system_kwargs['progress_dt'] = 0.
 
     return psets, system_stats, system_kwargs
 
 
-def flatten_results(result, raise_if_exceptions=False):
+def flatten_results(results):
     seen_pset_keys = set()
     seen_result_keys = set()
     ret = []
 
-    for r in result:
-        if 'exception' in r:
-            if raise_if_exceptions:
-                raise ValueError('Exception seen: '+r['exception'])
-            else:
-                continue
+    for r in results:
         if 'pset' in r:
             [seen_pset_keys.add(k) for k in r['pset'].keys()]
         if 'result' in r:
@@ -155,10 +142,11 @@ def make_pset_ids(psets):
     ret = []
     for pset in psets:
         pset = pset.copy()  # essentially a 2-level copy of the user's list
-        pset_id = str(uuid.uuid4())  # flatten object because of serialization problems downstream
-        pset_ids[pset_id] = pset
         if '_pset_id' in pset:
             print('pset already has a _pset_id:', pset)
+        pset_sans_id = pset.copy()
+        pset_id = str(uuid.uuid4())  # flatten object because of serialization problems downstream
+        pset_ids[pset_id] = pset_sans_id
         pset['_pset_id'] = pset_id
         ret.append(pset)
     return ret, pset_ids
@@ -166,17 +154,40 @@ def make_pset_ids(psets):
 
 def finalize_progress(system_kwargs):
     progress = system_kwargs['progress']
-    failures = progress.get('failures', 0)
+    failures = progress.failures
     actual_failures = len(system_kwargs['pset_ids'])
 
-    verbose = system_kwargs.get('verbose', 0)
-
     # needed to fixup wrapper failures
     if actual_failures > failures:
         print('correcting failure count from {} to {}'.format(failures, actual_failures), file=sys.stderr)
-        progress['failures'] = actual_failures
+        progress.failures = actual_failures
     elif actual_failures < failures:
         print('can\'t happen! missing pset_ids {} less than failures {}'.format(actual_failures, failures), file=sys.stderr)
     else:
-        if verbose > 1 and failures > 0:
+        if system_kwargs['verbose'] > 1 and failures > 0:
             print('failures equal to actual failures, hurrah', file=sys.stderr)
+
+
+def handle_return_common(out_func, ret, system_stats, system_kwargs, user_kwargs):
+    progress = system_kwargs['progress']
+    for user_ret, system_ret in ret:
+        if 'result' in user_ret and not isinstance(user_ret['result'], dict) and user_ret['result'] is not None:
+            # fake an exception, make this case look like other failures
+            user_ret['exception'] = "ValueError('user function did not return a dict: {}')".format(
+                repr(user_ret['result']))
+            user_ret['result'] = {}
+        if 'raw_stats' in system_ret:
+            system_stats.combine_stats(system_ret['raw_stats'])
+        pset_id = user_ret['pset']['_pset_id']
+        if 'exception' in user_ret:
+            progress.failures += 1
+            progress.exceptions += 1
+            system_kwargs['pset_ids'][pset_id]['exception'] = user_ret['exception']
+        else:
+            del system_kwargs['pset_ids'][pset_id]
+            system_kwargs['results'].append(user_ret)
+            progress.finished += len(ret)
+        if out_func:
+            out_func(user_ret, system_kwargs, user_kwargs)
+
+    report_progress(system_kwargs)
